#
#  This file is part of Documancer (http://documancer.sf.net)
#
#  Copyright (C) 2004-2005 Vaclav Slavik
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License version 2 as
#  published by the Free Software Foundation.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#  $Id: pylucene.py,v 1.6 2005/02/06 10:09:31 vaclavslavik Exp $
#
#  Fulltext indexer using PyLucene running in external process

import sys, os.path

import utils
from indexers import FulltextIndexer
from _external import ExternalIndexer

PyLuceneIndexer = None

# Running PyLucene in-process requires at least Python 2.4 and PyLucene 0.9.6
# because of threading issues with GCJ. Because importing PyLucene is not cheap,
# don't even both if Python is too old:

if sys.version_info >= (2,4):
    # try import PyLucene and see if it's new enough:
    import PyLucene
    verstr = PyLucene.VERSION.split('.')
    ver = (int(verstr[0]), int(verstr[1]), int(verstr[2]))
    if ver >= (0,9,6):
        import pylucene_impl
        PyLuceneIndexer = pylucene_impl.PyLuceneIndexer


if PyLuceneIndexer == None:
    # no suitable PyLucene found, use out-of-process version:

    class PyLuceneExternalIndexer(ExternalIndexer):
        """PyLucene-based fulltext indexer."""

        def __init__(self):
            self.port = utils.findUnusedPort()
            ExternalIndexer.__init__(self, 'http://localhost:%i' % self.port)
        
        def getStartCommand(self):
            script = os.path.join(utils.getDocumancerHome(),
                                  'indexers', 'pylucene_srv.py')

            binary = sys.executable
            if binary.lower().endswith('documancer.exe'):
                binary = binary.lower().replace('documancer.exe', 'python.exe')

            return [binary, script, str(self.port)]

    PyLuceneIndexer = PyLuceneExternalIndexer
